package com.syjy;

import cn.hutool.core.lang.Console;
import com.syjy.container.ProtocolDataContainer;
import com.syjy.tunnelinfo.BaseTunnelInfo;
import com.syjy.tunnelinfo.DataPoint;
import lombok.Data;
import org.slf4j.Logger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;


/**
 * 入库存表任务
 */
public class DataStorageTask {



    private DataStorageTask() {
    }

    /**
     * 获取单例实例
     *
     * @return the instance
     */
    public static final DataStorageTask getInstance() {
        return DataStorageTask.LazyHolder.INSTANCE;
    }

    private static class LazyHolder {
        private static final DataStorageTask INSTANCE = new DataStorageTask();
    }

    /**
     * 线程池
     */
    ScheduledExecutorService executor = Executors.newScheduledThreadPool(4);


    private  static  final ProtocolDataContainer instance = ProtocolDataContainer.getInstance();

    /**
     * 通道池
     */
    List<WindDataRecorder> recorders = new ArrayList<>();

    public void addRecorder(BaseTunnelInfo tunnelInfo) {
        Map<String,List<DataPoint>> eqPoints=new HashMap<>();
        String eqName="";
        for(DataPoint d:tunnelInfo.getDataPoints()){
            if(!d.getDescribe().contains(":")){
                throw new RuntimeException("点位描述必须为 “设备名:点释义”");
            }
            eqName= d.getDescribe().substring(0,d.getDescribe().indexOf(":"));
            if(eqPoints.containsKey(eqName)){
                eqPoints.get(eqName).add(d);
            }else{
                List<DataPoint> ds=new ArrayList<>();
                ds.add(d);
                eqPoints.put(eqName,ds);
            }
        }
        for(Map.Entry<String,List<DataPoint>> e :eqPoints.entrySet()){
            WindDataRecorder windDataRecorder = new WindDataRecorder(e.getValue(),e.getKey());
            recorders.add(windDataRecorder);
            executor.scheduleAtFixedRate(() -> {
                write2File(windDataRecorder);
            }, 0, 1, TimeUnit.SECONDS);
        }

    }


    private void write2File(WindDataRecorder windDataRecorder) {
        try {
            windDataRecorder.getWriter().info(instance.getNumber(windDataRecorder.getWsPoint()).toString()+","+instance.getNumber(windDataRecorder.getWdPoint()).toString());
        }catch (Exception e){
            Console.error(e);
        }
    }


    @Data
    static class WindDataRecorder {

        Integer wsPoint;

        Integer wdPoint;

        Logger writer;



        /**
         * 构造方法
         * @param
         */
        public WindDataRecorder(List<DataPoint> dataPoints,String eqName) {
            for(DataPoint d:dataPoints){
                if(d.getDescribe().contains("风速")){
                    this.wsPoint=d.getId();
                    instance.putDescribe(d.getId(),eqName+":风速");
                }else if(d.getDescribe().contains("风向")){
                    this.wdPoint=d.getId();
                    instance.putDescribe(d.getId(),eqName+":风向");
                }
            }
           this.writer= WriteDataCSVLoggerFactory.getLogger(eqName);
            writer.info("风速,风向");
        }


    }


}
