package com.syjy.tunnelworker.gathers.iml;


import com.syjy.DataExchangeException;
import com.syjy.tunnelinfo.TunnelStatus;
import com.syjy.tunnelinfo.gathertunnelinfo.GatherModbusTcpTunnelInfo;
import com.syjy.tunnelworker.BaseProtocolTunnel;
import com.syjy.tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import wei.yigulu.modbus.domain.FunctionCode;
import wei.yigulu.modbus.netty.HSModbusTcpMasterBuilder;

/**
 * modbus协议TCP数据采集
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

public class ModbusTcpDataGather extends AbstractModbusDataGather<GatherModbusTcpTunnelInfo, HSModbusTcpMasterBuilder> {

	/**
	 * 构造方法
	 *
	 * @param gatherModbusTcpTunnelInfo 通道信息
	 */
	public ModbusTcpDataGather(final GatherModbusTcpTunnelInfo gatherModbusTcpTunnelInfo) {
		super(gatherModbusTcpTunnelInfo);
	}

	@Override
	public ModbusTcpDataGather buildTunnel() throws DataExchangeException {
		protocolBuilder = new ModbusMaster(tunnelInfo.getRemoteIp(), tunnelInfo.getRemotePort());
		protocolBuilder.setSpareIp(tunnelInfo.getRemoteSpareIp());
		protocolBuilder.setSparePort(tunnelInfo.getRemoteSparePort());
		protocolBuilder.setSelfIp(tunnelInfo.getSelfIp());
		protocolBuilder.setSelfPort(tunnelInfo.getSelfPort());
		protocolBuilder.setLog(this.log);
		setSlaveId(this.tunnelInfo.getSlaveId());
		setFunctionCode(FunctionCode.valueOf(this.tunnelInfo.getFunctionCode()));
		setTunnelStatus(TunnelStatus.BUILT);
		protocolTunnelContainer.addTunnel(this);
		parseGatherDataPoint();
		log.info("成功创建ModbusTCPMaster通道对象：{}", this.tunnelInfo.getTunnelName());
		return this;
	}



	@Override
	public ModbusTcpDataGather startTunnel() throws DataExchangeException {
		log.info("modbus TCP master {} 通道开始连接", getTunnelInfo().getTunnelName());
		protocolTunnelContainer.addUpdateDateTask(this);
		try {
			SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> {
				try {
					this.protocolBuilder.create();
					log.error("modbus TCP master  创建通道失败");
					setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
				} catch (Exception e) {
					log.error("modbus TCP master  创建通道失败", e);
					setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
				}
			});
		} catch (Exception e) {
			log.error("modbus TCP master 创建通道失败", e);
			setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
			throw new DataExchangeException(10007, "modbus TCP master创建通道失败");
		}
		if (!TunnelStatus.LISTENPORTFAIL.equals(getTunnelStatus())) {
			setTunnelStatus(TunnelStatus.LISTENPORTSUCCESS);
		}
		return this;
	}


	@Override
	public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
		if (this.protocolBuilder != null) {
			this.protocolBuilder.stop();
		}
		log.info("关闭 modbus TCP master 通道 {}", this.tunnelInfo.getTunnelName());
		return super.tunnelStop();
	}


	@Override
	public void getDataFromProtocol() throws DataExchangeException {
		try {
			super.getDataFromProtocol();
			this.consecutiveExceptions=0;
			this.setTunnelStatus(TunnelStatus.CONNECTED);
		}catch (Exception e){
			this.consecutiveExceptions++;
			if(consecutiveExceptions>10) {
				this.setTunnelStatus(TunnelStatus.CONNECTEDANDCOMMERROR);
			}
		}
	}


	class ModbusMaster extends HSModbusTcpMasterBuilder {

		public ModbusMaster(String ip, Integer port) {
			super(ip, port);
		}

		@Override
		public void connected() {
			setTunnelStatus(TunnelStatus.CONNECTED);
		}

		@Override
		public void disconnected() {
			setTunnelStatus(TunnelStatus.LOSECONN);
		}
	}

}
